/**
 * This program has been developed by a student from the bachelor Information Science at
 * Utrecht University within the Research Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

import { CompositeLayer } from 'deck.gl/typed';
import Supercluster from 'supercluster';
import { ScatterplotLayer, LineLayer } from '@deck.gl/layers/typed';
import { LayerProps } from '../types';
import { getColorFromScale } from '../utils';
import { BrushingExtension } from '@deck.gl/extensions/typed';
import GeoEdge from '../core/edge';
import GeoNode from '../core/node';

export class GeoClusterLayer extends CompositeLayer<LayerProps> {
  /**
   * Layer that clusters the given graph
   * Cluster layer uses supercluster library to cluster nodes
   * Edges get updated accordingly
   */
  static componentName = 'Cluster Layer';

  shouldUpdateState({ changeFlags, props, oldProps }: { props: LayerProps; oldProps: LayerProps; changeFlags: any }) {
    return changeFlags.somethingChanged || oldProps.clusterRadius !== props.clusterRadius;
  }

  updateState({ props, oldProps, changeFlags }: { props: LayerProps; oldProps: LayerProps; changeFlags: any }) {
    const rebuildIndex = changeFlags.dataChanged;
    const zoom = Math.floor(this.context.viewport.zoom);
    const radiusChanged = oldProps.clusterRadius !== props.clusterRadius;

    if (rebuildIndex || radiusChanged) {
      const index = new Supercluster({
        radius: props.clusterRadius,
        maxZoom: 16,
      });
      index.load(
        props.graph.getNodes().map((node: GeoNode) => ({
          geometry: { coordinates: node.getCoordinates() },
          properties: node,
        }))
      );
      this.setState({ index });
    }

    if (rebuildIndex || zoom !== this.state.zoom) {
      const nodes = this.state.index.getClusters([-180, -85, 180, 85], zoom);

      // Create node index for faster coordinate search
      const node_index: { [key: string]: any } = {};
      for (const node of nodes) {
        const coordinates = node.geometry.coordinates;
        if (!node.properties.cluster) {
          node_index[node.properties.id] = {
            id: node.properties.id,
            coordinates: coordinates,
          };
        } else {
          const children = this.state.index.getLeaves(node.id, Infinity);
          for (const child of children) {
            if (!node_index[child.properties.id]) {
              node_index[child.properties.id] = {
                id: node.id,
                coordinates: coordinates,
              };
            }
          }
        }
      }

      // Add updated coordinates to edges
      const edges: { [key: string]: GeoEdge } = {};
      props.graph.getEdges().forEach((edge: GeoEdge) => {
        const source: { [key: string]: any } = node_index[edge.from];
        const target: { [key: string]: any } = node_index[edge.to];

        if (source && target) {
          if (source.coordinates[0] === target.coordinates[0] && source.coordinates[1] === target.coordinates[1]) {
            return;
          } else {
            const key = `${source.id}-${target.id}`;
            if (!edges[key]) {
              edges[key] = {
                ...edge,
                source: source.coordinates,
                target: target.coordinates,
                count: 1,
              };
            } else edges[key].count++;
          }
        }
      }, []);

      this.setState({
        nodes,
        edges: Object.values(edges),
        zoom,
      });
    }
  }

  renderLayers() {
    return [
      new LineLayer({
        id: 'edges',
        data: this.state.edges,
        pickable: true,
        //@ts-ignore
        extensions: [new BrushingExtension()],
        brushingEnabled: this.props.brushing,
        brushingRadius: 500000,
        brushingTarget: 'source_target',
        getWidth: (d: any) => Math.sqrt(d.count) * 0.3,
        getSourcePosition: (d: any) => d.source,
        getTargetPosition: (d: any) => d.target,
        getColor: (d: any) => getColorFromScale(d.data.prediction, this.props.colorScale),
      }),
      new ScatterplotLayer({
        id: 'nodes',
        data: this.state.nodes,
        opacity: 1,
        pickable: true,
        interactive: true,
        filled: true,
        radiusScale: 3,
        radiusMinPixels: 5,
        radiusMaxPixels: 20,
        getRadius: (d: any) => (d.hasOwnProperty('type') ? d.properties.point_count * 1000 : 10),
        getFillColor: (d: any) => (d.properties.cluster ? [250, 0, 0] : [0, 0, 255]),
        getPosition: (d: any) => d.geometry.coordinates,
      }),
    ];
  }
}
