/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import { select } from 'd3';
import React, { useEffect, useRef } from 'react';
import { HyperEdgeI } from '../types';
import CustomLine from './CustomLine';

type HyperEdgeRangeProps = {
  index: number;
  text: string;
  hyperEdges: HyperEdgeI[];
  nRows: number;
  nCols: number;
  colOffset: number;
  xOffset: number;
  yOffset: number;
  rowHeight: number;
  hyperedgeColumnWidth: number;
  gapBetweenRanges: number;
  textBothMargins: number;
  onMouseEnter: (col: number, nameToShow: string, colTable: number) => void;
  onMouseLeave: (col: number) => void;
};
export const HyperEdgeRange = (props: HyperEdgeRangeProps) => {
  // calculate the width of the hyperEdgeRange.
  const rectWidth = props.hyperEdges.length * props.hyperedgeColumnWidth + 2 * props.gapBetweenRanges;

  const rows: JSX.Element[] = [];
  for (let i = 0; i < props.nRows; i++)
    rows.push(
      <g key={i} className={'row-' + i}>
        <rect
          key={i.toString()}
          y={i * props.rowHeight}
          width={rectWidth}
          height={props.rowHeight}
          fill={i % 2 === 0 ? 'hsl(var(--clr-sec--50))' : 'hsl(var(--clr-sec--0))'}
          stroke={'white'}
          strokeWidth={0}
          style={{ zIndex: -1 }}
        />
        {i === 0 && <CustomLine x1={0} x2={rectWidth} y1={0} y2={0} strokeWidth={1} />}

        <CustomLine x1={0} x2={rectWidth} y1={(i + 1) * props.rowHeight} y2={(i + 1) * props.rowHeight} strokeWidth={1} />
        <CustomLine x1={rectWidth} x2={rectWidth} y1={i * props.rowHeight} y2={(i + 1) * props.rowHeight} strokeWidth={1} />
      </g>,
    );

  let hyperedges = props.hyperEdges.map((hyperEdge, i) => (
    <HyperEdge
      colTable={props.index}
      key={i}
      col={i + props.colOffset}
      connectedRows={hyperEdge.indices}
      numbersOnRows={hyperEdge.frequencies}
      nameToShow={hyperEdge.nameToShow}
      rowHeight={props.rowHeight}
      xOffset={props.hyperedgeColumnWidth / 2 + i * props.hyperedgeColumnWidth + props.gapBetweenRanges}
      radius={0.227 * props.rowHeight}
      onMouseEnter={props.onMouseEnter}
      onMouseLeave={props.onMouseLeave}
    />
  ));

  const xOffset = props.xOffset + (props.index * 3 + 1) * props.gapBetweenRanges + props.colOffset * props.hyperedgeColumnWidth;
  return (
    <g transform={'translate(' + xOffset + ', +' + props.yOffset + ')'}>
      <g
        className={'text-columns text-col-' + props.index}
        transform={`translate(${rectWidth / 2}, ${0 * props.rowHeight})rotate(-90, ${0}, ${0})`}
      >
        {/* vertical line */}
        <CustomLine x1={0} x2={props.yOffset} y1={-rectWidth / 2} y2={-rectWidth / 2} strokeWidth={1} />
        {props.index === props.nCols - 1 && (
          <CustomLine x1={0} x2={props.yOffset} y1={(1 * rectWidth) / 2} y2={(1 * rectWidth) / 2} strokeWidth={1} />
        )}
        {/* horizontal */}
        <CustomLine x1={props.yOffset} x2={props.yOffset} y1={-rectWidth / 2} y2={rectWidth / 2} strokeWidth={1} />
        <text
          x={props.textBothMargins}
          textAnchor="start"
          dominantBaseline="middle"
          className="font-sans font-black font-semibold"
          fill="hsl(var(--clr-sec--800))"
          style={{
            stroke: 'none',
          }}
        >
          {props.text !== 'No attribute' && props.text}
        </text>
      </g>
      <g className={'hyperEdgeBlock hyperEdge-col-' + props.index} transform={'translate(0,' + 0 + ')'}>
        {rows}
        {hyperedges}
      </g>
    </g>
  );
};

type HyperEdgeProps = {
  colTable: number;
  col: number;
  connectedRows: number[];
  numbersOnRows: number[];
  rowHeight: number;
  xOffset: number;
  radius: number;
  nameToShow: string;
  onMouseEnter: (col: number, nameToShow: string, colTable: number) => void;
  onMouseLeave: (col: number) => void;
};

const HyperEdge = (props: HyperEdgeProps) => {
  const ref = useRef<SVGGElement>(null);

  useEffect(() => {
    if (ref.current) {
      select(ref.current)
        .on('mouseover', () => props.onMouseEnter(props.col, props.nameToShow, props.colTable))
        .on('mouseout', () => props.onMouseLeave(props.col));
    }
  }, [ref.current]);

  const circles = props.connectedRows.map((row, i) => {
    return (
      <g key={'row' + row + ' col' + i}>
        <circle
          className={'hyperEdge-col' + props.colTable}
          cx={0}
          cy={row * props.rowHeight}
          r={props.radius}
          fill="hsl(var(--clr-white))"
          stroke="hsl(var(--clr-black))"
          strokeWidth={1}
        />
      </g>
    );
  });

  const lines: JSX.Element[] = [];
  let y1 = props.connectedRows[0] * props.rowHeight + props.radius;
  let y2;
  for (let i = 1; i < props.connectedRows.length; i++) {
    y2 = props.connectedRows[i] * props.rowHeight - props.radius;
    lines.push(<line key={'line' + i} x1={0} y1={y1} x2={0} y2={y2} stroke="hsl(var(--clr-black))" strokeWidth={props.rowHeight / 30} />);
    y1 = props.connectedRows[i] * props.rowHeight + props.radius;
  }

  const yOffset = props.rowHeight * 0.5;

  return (
    <g ref={ref} className={'col-' + props.col} transform={'translate(' + props.xOffset + ',' + yOffset + ')'}>
      {lines}
      {circles}
    </g>
  );
};
